"""Data models for Recallio API requests and responses."""
from __future__ import annotations

from dataclasses import dataclass, asdict
from typing import List, Optional


@dataclass
class MemoryWriteRequest:
    """Request body for `/api/Memory/write`."""

    userId: str
    projectId: str
    content: str
    consentFlag: bool
    tags: Optional[List[str]] = None

    def to_dict(self) -> dict:
        return asdict(self)


@dataclass
class MemoryRecallRequest:
    """Request body for `/api/Memory/recall`."""

    projectId: str
    userId: str
    query: str
    scope: str
    tags: Optional[List[str]] = None
    limit: Optional[int] = 10
    similarityThreshold: Optional[float] = 0.3
    summarized: Optional[bool] = False
    reRank: Optional[bool] = False
    type: Optional[str] = 'facts'

    def to_dict(self) -> dict:
        return asdict(self)

    def to_body(self) -> dict:
        return {
            "projectId": self.projectId,
            "userId": self.userId,
            "query": self.query,
            "scope": self.scope,
            "tags": self.tags,
        }

    def to_params(self) -> dict:
        params = {}
        if self.limit is not None:
            params["limit"] = self.limit
        if self.similarityThreshold is not None:
            params["similarityThreshold"] = self.similarityThreshold
        if self.summarized is not None:
            params["summarized"] = self.summarized
        if self.reRank is not None:
            params["reRank"] = self.reRank
        if self.type is not None:
            params["type"] = self.type
        return params


@dataclass
class MemoryDeleteRequest:
    """Request body for `/api/Memory/delete`."""

    scope: str
    userId: Optional[str] = None
    projectId: Optional[str] = None

    def to_dict(self) -> dict:
        return asdict(self)


@dataclass
class MemoryDto:
    """Response body for memory objects."""

    id: Optional[str] = None
    userId: Optional[str] = None
    projectId: Optional[str] = None
    content: Optional[str] = None
    tags: Optional[List[str]] = None
    createdAt: Optional[str] = None
    expiresAt: Optional[str] = None


@dataclass
class MemoryWithScoreDto(MemoryDto):
    """Memory object returned from search with similarity information."""

    similarityScore: Optional[float] = None
    similarityLevel: Optional[str] = None


@dataclass
class ErrorReturnClass:
    """Error response returned by the API."""

    error: str


@dataclass
class RecallSummaryRequest:
    """Request body for `/api/Memory/recall-summary`."""

    projectId: str
    userId: str
    scope: str
    tags: Optional[List[str]] = None

    def to_dict(self) -> dict:
        return asdict(self)


@dataclass
class SummarizedMemoriesDto:
    """Summarized memory response."""

    content: Optional[str] = None
    numberOfMemories: Optional[int] = None


@dataclass
class GraphAddRequest:
    """Request body for `/api/GraphMemory/add`."""

    data: Optional[str] = None
    user_id: Optional[str] = None
    project_id: Optional[str] = None

    def to_dict(self) -> dict:
        return asdict(self)


@dataclass
class GraphSearchRequest:
    """Request body for `/api/GraphMemory/search`."""

    query: Optional[str] = None
    user_id: Optional[str] = None
    project_id: Optional[str] = None
    limit: Optional[int] = None
    threshold: Optional[float] = None

    def to_dict(self) -> dict:
        return asdict(self)


@dataclass
class GraphSearchResult:
    """Result item returned from graph search."""

    source: Optional[str] = None
    relationship: Optional[str] = None
    destination: Optional[str] = None
    score: Optional[float] = None
    source_type: Optional[str] = None
    destination_type: Optional[str] = None


@dataclass
class MemoryExportRequest:
    """Query parameters for `/api/Memory/export`."""

    type: str
    format: str
    userId: Optional[str] = None
    projectId: Optional[str] = None
    startDate: Optional[str] = None
    endDate: Optional[str] = None

    def to_params(self) -> dict:
        params = {
            "Type": self.type,
            "Format": self.format,
        }
        if self.userId is not None:
            params["UserId"] = self.userId
        if self.projectId is not None:
            params["ProjectId"] = self.projectId
        if self.startDate is not None:
            params["StartDate"] = self.startDate
        if self.endDate is not None:
            params["EndDate"] = self.endDate
        return params



"""HTTP client for the Recallio API."""
from __future__ import annotations

import requests

from .models import (
    MemoryWriteRequest,
    MemoryRecallRequest,
    MemoryDeleteRequest,
    RecallSummaryRequest,
    MemoryExportRequest,
    MemoryDto,
    MemoryWithScoreDto,
    SummarizedMemoriesDto,
    GraphAddRequest,
    GraphSearchRequest,
    GraphSearchResult,
)
from .errors import RecallioAPIError


class RecallioClient:
    """Client for interacting with the Recallio API."""

    def __init__(self, api_key: str, base_url: str = "https://app.recallio.ai") -> None:
        self.api_key = api_key
        self.base_url = base_url.rstrip("/")
        self.session = requests.Session()
        self.session.headers.update(
            {
                "Authorization": f"Bearer {self.api_key}",
                "Content-Type": "application/json",
                "Accept": "application/json",
            }
        )

    def _request(
        self,
        method: str,
        path: str,
        json: dict | None = None,
        params: dict | None = None,
    ) -> dict:
        url = f"{self.base_url}{path}"
        response = self.session.request(method, url, json=json, params=params)
        if response.status_code >= 200 and response.status_code < 300:
            if response.content:
                try:
                    return response.json()
                except ValueError:
                    return response.text
            return {}
        try:
            data = response.json()
            message = data.get("error", response.text)
        except ValueError:
            message = response.text
        raise RecallioAPIError(message, status_code=response.status_code)

    def write_memory(self, request: MemoryWriteRequest) -> MemoryDto:
        data = self._request("POST", "/api/Memory/write", json=request.to_dict())
        return MemoryDto(**data)

    def recall_memory(self, request: MemoryRecallRequest) -> list[MemoryWithScoreDto]:
        data = self._request(
            "POST",
            "/api/Memory/recall",
            json=request.to_body(),
            params=request.to_params(),
        )
        if isinstance(data, list):
            return [MemoryWithScoreDto(**item) for item in data]
        return [MemoryWithScoreDto(**data)]

    def recall_summary(self, request: RecallSummaryRequest) -> SummarizedMemoriesDto:
        data = self._request("POST", "/api/Memory/recall-summary", json=request.to_dict())
        if isinstance(data, dict):
            return SummarizedMemoriesDto(**data)
        return SummarizedMemoriesDto()

    def delete_memory(self, request: MemoryDeleteRequest) -> None:
        self._request("DELETE", "/api/Memory/delete", json=request.to_dict())
        return None

    def export_memory(self, request: MemoryExportRequest) -> str:
        data = self._request("GET", "/api/Memory/export", params=request.to_params())
        if isinstance(data, str):
            return data
        return ""

    def add_graph_memory(self, request: GraphAddRequest) -> MemoryDto:
        data = self._request("POST", "/api/GraphMemory/add", json=request.to_dict())
        return MemoryDto(**data)

    def search_graph_memory(self, request: GraphSearchRequest) -> list[GraphSearchResult]:
        data = self._request("POST", "/api/GraphMemory/search", json=request.to_dict())
        if isinstance(data, list):
            return [GraphSearchResult(**item) for item in data]
        return [GraphSearchResult(**data)]

    def get_graph_relationships(
        self,
        user_id: str | None = None,
        project_id: str | None = None,
        limit: int | None = None,
    ) -> list[GraphSearchResult]:
        params = {}
        if user_id is not None:
            params["userId"] = user_id
        if project_id is not None:
            params["projectId"] = project_id
        if limit is not None:
            params["limit"] = limit
        data = self._request("GET", "/api/GraphMemory/relationships", params=params)
        if isinstance(data, list):
            return [GraphSearchResult(**item) for item in data]
        return [GraphSearchResult(**data)]

    def delete_all_graph_memory(
        self,
        user_id: str | None = None,
        project_id: str | None = None,
    ) -> None:
        params = {}
        if user_id is not None:
            params["userId"] = user_id
        if project_id is not None:
            params["projectId"] = project_id
        self._request("DELETE", "/api/GraphMemory/delete-all", params=params)
        return None



"""Recallio API client package."""

from .client import RecallioClient
from .models import (
    MemoryWriteRequest,
    MemoryRecallRequest,
    MemoryDeleteRequest,
    MemoryDto,
    MemoryWithScoreDto,
    RecallSummaryRequest,
    SummarizedMemoriesDto,
    GraphAddRequest,
    GraphSearchRequest,
    GraphSearchResult,
    MemoryExportRequest,
    ErrorReturnClass,
)
from .errors import RecallioAPIError

__all__ = [
    "RecallioClient",
    "MemoryWriteRequest",
    "MemoryRecallRequest",
    "MemoryDeleteRequest",
    "MemoryDto",
    "MemoryWithScoreDto",
    "RecallSummaryRequest",
    "SummarizedMemoriesDto",
    "GraphAddRequest",
    "GraphSearchRequest",
    "GraphSearchResult",
    "MemoryExportRequest",
    "ErrorReturnClass",
    "RecallioAPIError",
]



"""Custom exceptions for Recallio client."""

class RecallioAPIError(Exception):
    """Exception raised when the Recallio API returns an error response."""

    def __init__(self, message: str, status_code: int | None = None) -> None:
        self.message = message
        self.status_code = status_code
        super().__init__(f"{status_code}: {message}" if status_code else message)
